#include "RIPCTest.h"

/**
 * Test for deadlock detection
 * Usage:
 * Start N processes with "Deadlock [-nodeadlock] [-timed] N" command
 * Output: "Number of detected deadlocks: XXX"
 */
class MutexDeadlock : public RIPCTest { 
  public:
    void run(RIPCSession* session, int id) { 
	int nParticipants = nProcesses;
	int i;
	RIPCBarrier* barrier = session->createBarrier("barrier", nParticipants);
	RIPCMutex** mutex = new RIPCMutex*[nParticipants];
	for (i = 0; i < nParticipants; i++) { 
	    char name[64];
	    sprintf(name, "mutex%d", i);
	    mutex[i] = session->createMutex(name, false);
	}
	int nDeadlocks = 0;

	barrier->waitFor();
	for (i = 0; i < nIterations; i++) { 
	    int firstLock = rand() % nParticipants;
	    int secondLock = rand() % nParticipants;
	    if ((options & NO_DEADLOCK) && firstLock > secondLock) { 		
		int tmp = firstLock;
		firstLock = secondLock;
		secondLock = tmp;
	    }
	    if (options & TIMED_WAIT) { 
		mutex[firstLock]->lock();
		if (mutex[secondLock]->lock(timeout)) {
		    mutex[secondLock]->unlock();
		} else { 
		    nDeadlocks += 1;
		}
	    } else { 
		mutex[firstLock]->lock();
		try { 
		    mutex[secondLock]->lock();
		    mutex[secondLock]->unlock();
		} catch (RIPCDeadlockException const&) { 
		    nDeadlocks += 1;
		}
	    }
	    mutex[firstLock]->unlock();
	} 
	printf("Number of detected deadlocks: %d\n", nDeadlocks);
    }
};



int main(int argc, char* argv[]) { 
    MutexDeadlock test;
    return test.main(argc ,argv);
}




