#include "RIPCTest.h"

#define HELLO "Hello world"

/**
 * Test for mutex, barrier and shared memory
 * Usage: start N processes by "java Shmem N" command
 * Output: "Detect XXX shared object updates"
 */
class Shmem : public RIPCTest { 
  public:
    struct SharedObject {
	int    ivalue;
	char   svalue[64];
    };

    class SharedMemoryMonitor { 
      public:
	RIPCTest*  test;
	RIPCThread thread;
	int updateCount;
	int lastValue;
	
	SharedMemoryMonitor(RIPCTest* test, int lastValue) { 
	    this->test = test;
	    this->lastValue = lastValue;
	    updateCount = 0;
	}
	
	static void thread_proc startMonitor(void* arg) { 
	    ((SharedMemoryMonitor*)arg)->run();
	}

	void start() { 
	    thread.start(&startMonitor, this);
	}

	void wait() { 
	    thread.join();
	}

        void run() { 
	    RIPCSession* session = test->getSession();
	    RIPCSharedMemory* shmem = session->openSharedMemory("shmem");
	    if (shmem != NULL) { 
		while (true) {  
		    if (session->isLocal()) { 
			shmem->waitFor(test->timeout);	    
		    } else { 
			shmem->waitFor();	    
		    }
		    SharedObject so;
		    RIPCObject objDesc(&so, sizeof so);
		    shmem->get(objDesc);
		    updateCount += 1;
		    assert(strcmp(so.svalue, HELLO) == 0);
		    if (so.ivalue == lastValue) { 
			delete session;
			break;
		    }
		}
	    }
	}
    };

    void run(RIPCSession* session, int id) 
    {
	SharedObject so;
	so.ivalue = 0;
	strcpy(so.svalue, HELLO);

	RIPCMutex* mutex = session->createMutex("shmem", false);
	RIPCBarrier* barrier = session->createBarrier("shmem", nProcesses);
	RIPCSharedMemory* shmem = session->createSharedMemory("shmem", &so, sizeof(so));

	SharedMemoryMonitor* monitor = new SharedMemoryMonitor(this, nProcesses*nIterations);
	monitor->start();

	barrier->waitFor();
	for (int i = 0; i < nIterations; i++) { 	    
	    RIPCObject objDesc(&so, sizeof so);
	    mutex->lock();
	    shmem->get(objDesc);
	    so.ivalue += 1;
	    shmem->set(&so, sizeof so);
	    mutex->unlock();
	}
	barrier->waitFor();
	SharedObject* sp = (SharedObject*)shmem->get();
	assert(sp->ivalue == nIterations*nProcesses);
	assert(strcmp(sp->svalue, HELLO) == 0);
	shmem->deallocate(sp);
	monitor->wait();
	printf("Detect %d shared object updates\n", monitor->updateCount);
    }
};




int main(int argc, char* argv[]) { 
    Shmem test;
    return test.main(argc ,argv);
}






